/*
 * BlogView.cpp
 *
 *  Created on: 5.11.2009
 *      Author: thyv
 */

#include <QMessageBox>
#include <QMenu>
#include <QMenuBar>
#include <QAction>
#include <QStackedWidget>
#include <QFileSystemModel>
#include <QDesktopWidget>
#include <QResizeEvent>

#ifdef Q_OS_SYMBIAN
#include <aknappui.h>
#include <avkon.rsg>
#endif

#include "setupblog.h"
#include "BlogSetupView.h"
#include "BlogView.h"
#include "PostsView.h"
#include "PagesView.h"
#include "commentsview.h"
#include "addpostview.h"
#include "SinglePostView.h"
#include "CategoryView.h"
#include "writepageview.h"
#include "WPNetworkEngine.h"
#include "textlistitem.h"
#include "ImageSelectionList.h"
#include "ProgressDialog.h"
#include "DialogController.h"
#include "WPLogger.h"
#include "statisticsview.h"

BlogView::BlogView()
{
	// Workaround for issue of starting the app in landscape mode.
	// See the resizeEvent for more.
#ifdef Q_WS_MAEMO_5
	setAttribute(Qt::WA_Maemo5StackedWindow);
	setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
#endif
	iLandscapeSizeWorkaround = false;
	
	WPLogger::Instance().LogRow("Application started");
	
	iNetworkEngine = NULL;
	iImageSelectionList = NULL;	
	iBlogView = new BlogSetupView(this);
#ifdef Q_WS_MAEMO_5
	// This is required to ensure the functionality of the back button
	iSetupBlog = new SetupBlog(this, iBlogView);
#else
	iSetupBlog = new SetupBlog(this);
#endif
	iWritePageView = new WritePageView(this);
	iAddPostView = new AddPostView(this);
	iSinglePostView = new SinglePostView(*this);
	iDialogController = new DialogController(this);
        iStatisticsView = new StatisticsView(this);

#ifdef Q_WS_MAEMO_5
	iCategoryView = new CategoryView(this, iAddPostView);
#else
	iCategoryView = new CategoryView(this);
#endif

	iViewTitle << tr("Edit blog")
			<< tr("Blogs")
			<< tr("Edit page")
			<< tr("Edit post")
			<< tr("Post")
			<< tr("Categories")
                        << tr("Statistics")
			<< tr("WordPress")
			<< tr("Images");
	
	// The order in which the stacked widget is initialized
	// is significant -- the enums have to match
	iStackedWidget = new QStackedWidget(this);

#ifndef Q_WS_MAEMO_5
	iStackedWidget->insertWidget(ESetupBlogView, iSetupBlog);
	iStackedWidget->insertWidget(EBlogListView, iBlogView);
	iStackedWidget->insertWidget(EWritePageView, iWritePageView);
	iStackedWidget->insertWidget(EAddPostView, iAddPostView);
	iStackedWidget->insertWidget(ESinglePostView, iSinglePostView);
	iStackedWidget->insertWidget(ECategoryView, iCategoryView);
        iStackedWidget->insertWidget(EStatisticsView, iStatisticsView);
#endif
	
	iCommentsView = new CommentsView(*this);
	iPostsView = new PostsView(this);
	iPagesView = new PagesView(this);

	// Tabbed view
	iTabView = new QTabWidget(this);
	iTabView->setContextMenuPolicy(Qt::NoContextMenu);
	iTabView->setFocusPolicy(Qt::NoFocus);

#ifdef Q_WS_MAEMO_5
	// Some aligment bug in maemo, extra spaces fixes it.
	iTabView->addTab(iCommentsView, tr("  Comments  "));
	iTabView->addTab(iPostsView, tr("  Posts  "));
	iTabView->addTab(iPagesView, tr("  Pages  "));
#else
	iTabView->addTab(iCommentsView, tr("Comments"));
	iTabView->addTab(iPostsView, tr("Posts"));
	iTabView->addTab(iPagesView, tr("Pages"));
#endif

#ifdef Q_OS_SYMBIAN
	iTabView->setTabPosition(QTabWidget::North);
#else
	iTabView->setTabPosition(QTabWidget::West);
#endif
	
	iStackedWidget->insertWidget(ETabbedView, iTabView);
	iNetworkEngine = new WPNetworkEngine;
	
	// Blogs are found
	connect(iNetworkEngine, SIGNAL(GetUserBlogsCompleted(const WPDataBlog&)), this, SLOT(ShowTabbedView()));

	// Progress dialog
	connect(iNetworkEngine, SIGNAL(TaskStarted(const TWPNetworkEngineCommand&)), iDialogController, SLOT(TaskStarted(const TWPNetworkEngineCommand&)));
	connect(iNetworkEngine, SIGNAL(TaskFinished(const TWPNetworkEngineCommand&)), iDialogController, SLOT(TaskFinished(const TWPNetworkEngineCommand&)));
	connect(iNetworkEngine, SIGNAL(ShowErrorNote(const QString&)), iDialogController, SLOT(ShowErrorNote(const QString&)));
	
	// Local data is updated, update views
	connect(iAddPostView, SIGNAL(UpdatePostData(const WPDataBlog&)), iPostsView, SLOT(UpdatePostData(const WPDataBlog&)));
	connect(iWritePageView, SIGNAL(UpdatePageData(const WPDataBlog&)), iPagesView, SLOT(UpdatePageData(const WPDataBlog&)));
	
	// New data from network, update views
	connect(iNetworkEngine, SIGNAL(UpdateCategoryData(const WPDataBlog&)), iCategoryView, SLOT(UpdateCategoryData(const WPDataBlog&)));
	connect(iNetworkEngine, SIGNAL(UpdateData(const WPDataUsersBlogs&)), iBlogView, SLOT(UpdateData(const WPDataUsersBlogs&)));
	connect(iNetworkEngine, SIGNAL(UpdateCommentsData(const WPDataBlog&)), iCommentsView, SLOT(UpdateCommentsData(const WPDataBlog&)));
	connect(iNetworkEngine, SIGNAL(UpdatePostData(const WPDataBlog&)), iPostsView, SLOT(UpdatePostData(const WPDataBlog&)));
	connect(iNetworkEngine, SIGNAL(UpdateCommentsData(const WPDataBlog&)), iPostsView, SLOT(UpdateCommentsData(const WPDataBlog&)));
	connect(iNetworkEngine, SIGNAL(UpdatePageData(const WPDataBlog&)), iPagesView, SLOT(UpdatePageData(const WPDataBlog&)));
	connect(iNetworkEngine, SIGNAL(UpdateData(const WPDataBlog&)), iSinglePostView, SLOT(UpdateData(const WPDataBlog&)));

	// Connect page view context menu signals
	connect(iPagesView, SIGNAL(OpenClicked(TextListItem *)), this, SLOT(ShowEditPageView(TextListItem *)));
	connect(iPagesView, SIGNAL(EditClicked(TextListItem *)), this, SLOT(ShowEditPageView(TextListItem *)));
	connect(iPagesView, SIGNAL(DeleteClicked(TextListItem *)), this, SLOT(DeletePage(TextListItem *)));
	
	// Page editing view
	connect(iWritePageView, SIGNAL(EditPage(const QString&, const QMap<QString, QString>&, const QString&, const QStringList&)),
			iNetworkEngine,   SLOT(EditPage(const QString&, const QMap<QString, QString>&, const QString&, const QStringList&)));
	connect(iWritePageView, SIGNAL(NewPage(const QMap<QString, QString>&, const QString&, const QStringList&, const QString&)),
			iNetworkEngine,   SLOT(NewPage(const QMap<QString, QString>&, const QString&, const QStringList&, const QString&)));

	// Connect post view context menu signals
	connect(iPostsView, SIGNAL(OpenClicked(TextListItem *)), this, SLOT(ShowSinglePostView(TextListItem *)));
	connect(iPostsView, SIGNAL(EditClicked(TextListItem *)), this, SLOT(ShowEditPostView(TextListItem *)));
	connect(iPostsView, SIGNAL(DeleteClicked(TextListItem *)), this, SLOT(DeletePost(TextListItem *)));
	connect(iCategoryView, SIGNAL(NewCategory(const QString&)), iNetworkEngine, SLOT(NewCategory(const QString&)));
	
	// Post editing view
	connect(iAddPostView, SIGNAL(ShowErrorNote(const QString&)), iDialogController, SLOT(ShowErrorNote(const QString&)));
	connect(iAddPostView, SIGNAL(EditPost(const QString&, const QMap<QString, QString>&, const QMap<QString, QStringList>&, const QString&, const QStringList&)),
			iNetworkEngine, SLOT(EditPost(const QString&, const QMap<QString, QString>&, const QMap<QString, QStringList>&, const QString&, const QStringList&)));
	connect(iAddPostView, SIGNAL(NewPost(const QMap<QString, QString>&, const QMap<QString, QStringList>&, const QString&, const QStringList&, const QString&)),
			iNetworkEngine, SLOT(NewPost(const QMap<QString, QString>&, const QMap<QString, QStringList>&, const QString&, const QStringList&, const QString&)));

	// If local data is found, go directly to the tabbed view and
	// refresh data from the network. Otherwise, go to blog setup.
	if (iNetworkEngine->LoadLocalData()) {
		setCentralWidget(iStackedWidget);
		iNetworkEngine->RefreshAll();
		ShowTabbedView();
	} else {
		// At first run, the setup blog view already contains default
		// values, so just switch to the view directly.
#ifdef Q_WS_MAEMO_5
		// In Maemo, the central widget is first blog setup,
		// and will be changed to iStackedWidget once setup is done.
		iInitialSetupBlog = new SetupBlog(this);
		setCentralWidget(iInitialSetupBlog);
		SwitchView(EInitialSetupBlogView);
#else
		setCentralWidget(iStackedWidget);
		SwitchView(ESetupBlogView);
#endif
	}
}

BlogView::~BlogView()
{
	if (iNetworkEngine)
		iNetworkEngine->SaveAllData();
	delete iNetworkEngine;
	WPLogger::Instance().LogRow("Application exiting.");
	WPLogger::Instance().WriteFile();
}

void BlogView::SetSoftKeys()
{
	// Menu bar
	// TODO implement in a way that allows the same menu for all views

	QMenuBar* menubar = menuBar();
	menubar->clear();

	// Blogs submenu
	QAction* manageBlogs = new QAction(tr("Blogs"), this);
	connect(manageBlogs, SIGNAL(triggered()), this, SLOT(ShowBlogListView()));
	menubar->addAction(manageBlogs);
	
	// Other actions
	QAction* actionWritePage = new QAction(tr("Write new page"), this);
	connect(actionWritePage, SIGNAL(triggered()), this, SLOT(ShowWriteNewPageView()));
	menubar->addAction(actionWritePage);

	QAction* actionWritePost = new QAction(tr("Write new post"), this);
	connect(actionWritePost, SIGNAL(triggered()), this, SLOT(ShowWriteNewPostView()));
	menubar->addAction(actionWritePost);

	QAction* actionRefresh = new QAction(tr("Refresh"), this);
	connect(actionRefresh, SIGNAL(triggered()), this, SLOT(Refresh()));
	menubar->addAction(actionRefresh);
	
	QAction* about = new QAction(tr("About"), this);
	connect(about, SIGNAL(triggered()), this, SLOT(About()));
	menubar->addAction(about);
	
	QAction* exit = new QAction(tr("Exit"), this);
	connect(exit, SIGNAL(triggered()), this, SLOT(Exit()));
	menubar->addAction(exit);
}

void BlogView::ShowEditPageView(TextListItem *aItem)
{
	const QString& pageId = aItem->ItemId();
	if (!pageId.isNull())
	{
		iWritePageView->EditPage(*iNetworkEngine->GetCurrentBlog(), pageId);
		SwitchView(EWritePageView);
	}
}

void BlogView::ShowWriteNewPageView()
{
	iWritePageView->NewPage(*iNetworkEngine->GetCurrentBlog());
	SwitchView(EWritePageView);
}

void BlogView::ShowSinglePostView(TextListItem *aItem)
{
	const QString& postId = aItem->ItemId();
	if (!postId.isNull())
	{
		WPDataPost* post = iNetworkEngine->GetPost(postId);
		if (!post) post = iNetworkEngine->GetLocalDraftPost(postId);
		if (post)
		{
			iSinglePostView->ShowPost(*iNetworkEngine->GetCurrentBlog(), *post);
			SwitchView(ESinglePostView);
		}
	}
}

void BlogView::ShowEditPostView(TextListItem *aItem)
{
	// If no post is defined, we just switch to the view
	// and expect all data to be in place 
	if (aItem)
	{
		const QString& postId = aItem->ItemId();
		if (!postId.isNull())
		{
			WPDataPost* post = iNetworkEngine->GetPost(postId);
			if (!post) post = iNetworkEngine->GetLocalDraftPost(postId);
			if (post)
			{
				iAddPostView->EditPost(iNetworkEngine->GetCurrentBlog(), post);
			}
		}
	}

	SwitchView(EAddPostView);

}

void BlogView::ShowWriteNewPostView()
{
	iAddPostView->NewPost(iNetworkEngine->GetCurrentBlog());
	SwitchView(EAddPostView);
}

void BlogView::ShowTabbedView()
{
	SetSoftKeys();
    SwitchView(ETabbedView);
}

void BlogView::ShowBlogListView()
{
	// Update data so that latest activity is shown correctly
	iBlogView->UpdateData(*iNetworkEngine->GetAllBlogs());
	SwitchView(EBlogListView);
}

void BlogView::ShowNewBlogView()
{
	iSetupBlog->NewBlog();
	SwitchView(ESetupBlogView);
}

void BlogView::ShowEditBlogView(const QString& aBlogId)
{
	iSetupBlog->EditBlog(iNetworkEngine->GetBlog(aBlogId));
	SwitchView(ESetupBlogView);
}

void BlogView::ShowStatisticsView() {

}

void BlogView::SwitchView(const TMainView& aView)
{
#ifndef Q_WS_MAEMO_5
	setWindowTitle(iViewTitle.at(aView));
	iStackedWidget->setCurrentIndex(aView);
#else
	switch(aView)
	{
	case EInitialSetupBlogView:
		break;
	case ESetupBlogView:
		iSetupBlog->show();
		break;
	case EBlogListView:
		iBlogView->show();
		break;
	case EAddPostView:
		iAddPostView->show();
		break;
	case ESinglePostView:
		iSinglePostView->show();
		break;
	case EWritePageView:
		iWritePageView->show();
		break;
	case ECategoryView:
		iCategoryView->show();
		break;
	case EImageSelectionList:
		iImageSelectionList->show();
		break;
	case ETabbedView:
		//No action needed. All other "Views" uses Close() so this will be enabled by platform.
		break;
        case EStatisticsView:
                iStatisticsView->show();
	};
	// Check if we're swithing away from initial setup blog view. That view can't be
	// accessed again without restarting the application, so we'll delete it here.
	if (iCurrentView == EInitialSetupBlogView)
	{
		// QMainWindow has the ownership of the iInitialSetupBlog and
		// will take care of the deletion.
		setCentralWidget(iStackedWidget);
	}
#endif
	iCurrentView = aView;
	SetActiveView();
}

void BlogView::SaveBlogSetup(const QString& aBlogUrl, const QString& aUsername,
							 const QString& aPassword, const QString& aNumberOfPosts, bool aResizePhotos)
{
	iNetworkEngine->SaveBlogSetup(aBlogUrl, aUsername, aPassword, aNumberOfPosts, aResizePhotos);
}

void BlogView::DeletePage(TextListItem* aItem)
{
	iNetworkEngine->DeletePage(aItem->ItemId());
}

void BlogView::DeletePost(TextListItem* aItem)
{
	iNetworkEngine->DeletePost(aItem->ItemId(), "1");
}

void BlogView::OpenBlog(const QString& aBlogId)
{
	iNetworkEngine->OpenBlog(aBlogId);
	ShowTabbedView();
}

bool BlogView::DeleteBlog(const QString& aBlogId)
{
	return iNetworkEngine->DeleteBlog(aBlogId);
}

void BlogView::ShowCategoryView(const QStringList* aCategories)
{
	iCategoryView->SelectCategories(aCategories);
	SwitchView(ECategoryView);
}

void BlogView::SelectCategories(const QStringList* aCategories)
{
	iAddPostView->SelectCategories(aCategories);
	SwitchView(EAddPostView);
}

void BlogView::ShowImageList(const QStringList& aInitialSelections, bool aPostView)
{
	iAddPhotosToPosts = aPostView;
#ifndef Q_WS_MAEMO_5
	if (iImageSelectionList)
	{
		iStackedWidget->removeWidget(iImageSelectionList);
	}
#endif
	delete iImageSelectionList;
	iImageSelectionList = NULL;
#ifdef Q_WS_MAEMO_5
	if (aPostView)
	{
		iImageSelectionList = new ImageSelectionList(aInitialSelections, iAddPostView);
	}
	else
	{
		iImageSelectionList = new ImageSelectionList(aInitialSelections, iWritePageView);
	}
#else
	iImageSelectionList = new ImageSelectionList(aInitialSelections, this);
	iStackedWidget->insertWidget(EImageSelectionList, iImageSelectionList);
#endif

	connect(iImageSelectionList, SIGNAL(ImagesSelected(QStringList)), this, SLOT(ImagesSelected(QStringList))); // TODO change to function call, no point to use connect in this close classes
	connect(iImageSelectionList, SIGNAL(TaskStarted(const TWPNetworkEngineCommand&)), iDialogController, SLOT(TaskStarted(const TWPNetworkEngineCommand&)));
	connect(iImageSelectionList, SIGNAL(TaskFinished(const TWPNetworkEngineCommand&)), iDialogController, SLOT(TaskFinished(const TWPNetworkEngineCommand&)));

	// Call this only after the connections so that we can show a progess dialog
	iImageSelectionList->SearchImages();
	SwitchView(EImageSelectionList);
}

void BlogView::ImagesSelected(QStringList aFilenames)
{
	if (iAddPhotosToPosts)
	{
		iAddPostView->ImagesAdded(aFilenames);
		SwitchView(EAddPostView);
	}
	else
	{
		iWritePageView->ImagesAdded(aFilenames);
		SwitchView(EWritePageView);
	}
}

void BlogView::AddNewComment(const QString& aPostId, const QString& aBody)
{
	iNetworkEngine->NewComment(aBody, aPostId, 0);
	SwitchView(ESinglePostView);
}

void BlogView::ReplyToComment(const WPDataComment& aComment)
{
	// These can't be in local drafts
	WPDataPost* post = iNetworkEngine->GetPost(aComment.iPostId);
	if (post)
	{
		iSinglePostView->ShowPost(*iNetworkEngine->GetCurrentBlog(), *post);
		SwitchView(ESinglePostView);
	}
}

void BlogView::Exit()
{
	QCoreApplication::exit(0);
}

void BlogView::About()
{
	// TODO add an icon
	iDialogController->ShowInfoNote("About", "Nokia WordPress\nversion 0.5.4");
}

void BlogView::Refresh()
{
	iNetworkEngine->RefreshAll(true);
}

/**
 * Sets the active view after ProgressDialog closes. 
 */
void BlogView::SetActiveView(BlogView::TMainView aRefocusableView)
{
#ifndef Q_WS_MAEMO_5
	// Update focus if
	//   a) there is an active view AND
	//   b) either the call is general, or a specific view wants its focus
	//      to be updated in case it is the current view (separate views do
	//      not know if they are active at a given time or not).
	QWidget* currentView = iStackedWidget->widget(iCurrentView);
	if  (currentView != 0 && (aRefocusableView == BlogView::ENoView || aRefocusableView == BlogView::iCurrentView))
	{
		// TODO non-virtual setFocus calls base class function if called from
		// QWidget* -- create some Focusable interface to fix this? For now,
		// call separately for each view that has special focus function.
		if (iCurrentView == ETabbedView)
		{
			switch (iTabView->currentIndex())
			{
			case 0:
				iCommentsView->setFocus();
				break;
			case 1:
				iPostsView->setFocus();
				break;
			case 2:
				iPagesView->setFocus();
				break;
			}
		}
		else if (iCurrentView == ECategoryView)
			iCategoryView->setFocus();
		else if (iCurrentView == ESetupBlogView)
			iSetupBlog->setFocus();
		else if (iCurrentView == EBlogListView)
			iBlogView->setFocus();	
		else if (iCurrentView == EAddPostView)
			iAddPostView->setFocus();
		else if (iCurrentView == EWritePageView)
			iWritePageView->setFocus();
		else if (iCurrentView == EImageSelectionList && iImageSelectionList)
			iImageSelectionList->setFocus();
		else
		{
			currentView->setFocus();
		}
		
		// After calling setFocus, the options menu receives an extra "Actions" item.
		// It's not BlogView, it's not currentView. We'll get around it for now
		// by disabling context menu in all widgets.
		QList<QWidget*> list = QApplication::allWidgets();
		const int count = list.count();
		for (int i = 0; i < count; i++)
		{
			list.at(i)->setContextMenuPolicy(Qt::NoContextMenu);
		}
	}
#endif
}

void BlogView::resizeEvent(QResizeEvent *e)
{
	QSize window = QApplication::desktop()->screenGeometry().size();

#ifdef Q_OS_SYMBIAN
	// In portrait Symbian phones, let's use smaller status pane than normal
	// to get more pixels for the content
    if (window.width() < window.height()) // Portrait mode
    {
        CEikStatusPane* statusPane = static_cast<CAknAppUi*>(CEikonEnv::Static()->EikAppUi())->StatusPane();
        statusPane->SwitchLayoutL(R_AVKON_STATUS_PANE_LAYOUT_USUAL_FLAT);
    }
#endif

    // Workaround for landscape mode height exceeding the available height.
    // The pixel amount is hardcoded, found out manually.
    // TODO If a Qt fix becomes available, disable this.
	if (!iLandscapeSizeWorkaround)
	{
		// The maximum height is only set in the first resize event in landscape mode.
		// This seems to be the only situation when the application gets the wrong height.
		if (window.width() == 640 && window.height() == 360)
			iStackedWidget->setMaximumHeight(geometry().size().height()); // 29 -- 85
		else if (window.width() == 320 && window.height() == 240)
			iStackedWidget->setMaximumHeight(geometry().size().height()); // 57 -- 107
	}
	else if (iStackedWidget->maximumHeight() < QWIDGETSIZE_MAX)
	{
		// In further resize events, clear all restrictions
		iStackedWidget->setMaximumHeight(QWIDGETSIZE_MAX);
	}

	iLandscapeSizeWorkaround = true;
	e->ignore();
}
