/*
 * BaseView.h
 *
 *  Created on: 5.11.2009
 *      Author: thyv
 */

#ifndef BLOGVIEW_H_
#define BLOGVIEW_H_

#include <QMainWindow>
#include "WPData.h"

class QListWidgetItem;
class QStackedWidget;
class QTabWidget;
class QMenuBar;
class QAction;
class QFileSystemModel;

class WritePageView;
class AddPostView;
class SinglePostView;
class CategoryView;
class StatisticsView;

class PostsView;
class PagesView;
class CommentsView;
class SetupBlog;
class BlogSetupView;
class WPNetworkEngine;
class TextListItem;
class ImageSelectionList;
class DialogController;

class BlogView : public QMainWindow
{

	Q_OBJECT

public:
	BlogView();
	~BlogView();
	
	// The order is significant
	enum TMainView
	{
		EInitialSetupBlogView = -2, // Maemo specific, so the index doesn't have to be positive
		ENoView               = -1,
		ESetupBlogView        =  0,
		EBlogListView         =  1,
		EWritePageView        =  2,
		EAddPostView          =  3,
		ESinglePostView       =  4,
		ECategoryView         =  5,
                EStatisticsView       =  6,
                ETabbedView           =  7,
                EImageSelectionList   =  8 // This should be the last one as it is created dynamically
	};
	
	QStringList iViewTitle;
	
	WPNetworkEngine* GetNetworkEngine() { return iNetworkEngine; }
	void SwitchView(const TMainView& aView);
	void SaveBlogSetup(const QString& aBlogUrl, const QString& aUsername,
			const QString& aPassword, const QString& aNumberOfPosts, bool aResizePhotos);
	
	void AddNewComment(const QString& aPostId, const QString& aBody);
	QTabWidget* TabView() {return iTabView;}
	void ShowCategoryView(const QStringList* aCategories);
	void SelectCategories(const QStringList* aCategories);
	void AddNewCategory(const QString& aCategoryName);
	void ShowImageList(const QStringList& aInitialSelections, bool aPostView); // Pass true if post view, false if pages view
	void SetActiveView(BlogView::TMainView aRefocusableView = BlogView::ENoView);
	
signals:
	void SignalBlogListView();

public slots:
	void ShowTabbedView();
	void Exit();
	void Refresh();
	
	// Pages
	void ShowEditPageView(TextListItem *aItem);	
	void ShowWriteNewPageView();
	void DeletePage(TextListItem* aItem);

	// Posts
	void ShowEditPostView(TextListItem *aItem = 0);
	void ShowSinglePostView(TextListItem *aItem);
	void ShowWriteNewPostView();
	void DeletePost(TextListItem* aItem);
	void ReplyToComment(const WPDataComment& aComment);
	void ImagesSelected(QStringList aFilenames);
	
	// Blogs
	void ShowBlogListView();
	void ShowNewBlogView();
	void ShowEditBlogView(const QString& aBlogId);
	void OpenBlog(const QString& aBlogId);
	bool DeleteBlog(const QString& aBlogId);
	
        // Statistics
        void ShowStatisticsView();

public: // New functions
	WPNetworkEngine* GetNetworkEngine() const;

private:
	void SetSoftKeys();
	
private slots:
	void About();
	void resizeEvent(QResizeEvent *e);
	
private:
	// Views
	QStackedWidget* iStackedWidget;
	QTabWidget* iTabView;
	PostsView* iPostsView;
	PagesView* iPagesView;
	CommentsView* iCommentsView;
	SetupBlog* iSetupBlog;
	BlogSetupView* iBlogView;
#ifdef Q_WS_MAEMO_5
	SetupBlog* iInitialSetupBlog;
#endif
	WritePageView* iWritePageView;
	AddPostView* iAddPostView;
	SinglePostView* iSinglePostView;
	CategoryView* iCategoryView;
        StatisticsView* iStatisticsView;
	ImageSelectionList* iImageSelectionList;
	DialogController* iDialogController;
	
	// Network
	WPNetworkEngine* iNetworkEngine;

	int iCurrentView;
	bool iLandscapeSizeWorkaround;
	bool iAddPhotosToPosts; // True if post view, false if pages view
};
#endif //BLOGVIEW_H_
