/*
 * DialogController.cpp
 *
 *  Created on: 5.1.2010
 *      Author: apoi
 */

#include <QList>
#include <QTimer>

#include "DialogController.h"
#include "BlogView.h"
#include "ErrorDialog.h"
#include "ProgressDialog.h"

DialogController::DialogController(BlogView *aParent) : QObject(aParent)
{
	iParent = aParent;
	iProgress = new ProgressDialog(aParent, this);
	iError = new ErrorDialog(aParent, this);
	
	iCommandMessages << tr("Preparing")
		<< tr("Fetching main page")
		<< tr("Fetching blog RSD")
		<< tr("Fetching user's blogs")
		<< tr("Fetching pages")
		<< tr("Fetching posts")
		<< tr("Fetching categories")
		<< tr("Fetching tags")
		<< tr("Fetching page statuses")
		<< tr("Fetching post statuses")
		<< tr("Fetching comment statuses")
		<< tr("Fetching comments")
		<< tr("Adding a new page")
		<< tr("Editing a page")
		<< tr("Deleting a page")
		<< tr("Adding a new comment")
		<< tr("Editing a comment")
		<< tr("Deleting a comment")
		<< tr("Adding a new post")
		<< tr("Editing a post")
		<< tr("Deleting a post")
		<< tr("Fetching Gravatar")
		<< tr("Adding a new category")
		<< tr("Uploading a file");
}

DialogController::~DialogController()
{
}

void DialogController::ShowErrorNote(const QString& aError)
{
	iError->SetTitle(tr("Error occurred"));
	if (iProgress->isVisible())
		iProgress->hide();
	iError->ShowErrorNote(aError);
	iError->MoveAndShow();
}

void DialogController::ShowInfoNote(const QString& aTitle, const QString& aError)
{
	ShowErrorNote(aError);
	iError->SetTitle(aTitle);
}

void DialogController::TaskStarted(const TWPNetworkEngineCommand& aCommand)
{
	iCommandList.append(aCommand);
	if (!iError->isVisible())
	{
		SetProgressMessage();
		iProgress->MoveAndShow();
	}
}

void DialogController::TaskFinished(const TWPNetworkEngineCommand& aCommand)
{
	iCommandList.removeOne(aCommand);
	// If there's no more actions, close the dialog. However, use a small
	// delay, since the next network task might start just after this one
	// has ended -- this way we avoid progress bar flicking in and out.
	QTimer::singleShot(100, this, SLOT(DelayedProgressDialogClose()));
}

void DialogController::ExitProgress()
{
	iParent->Exit();
}

void DialogController::ErrorAccepted()
{
	iError->hide();
	if (iCommandList.count() > 0)
		iProgress->MoveAndShow();
	else
		iParent->SetActiveView();
}

void DialogController::SetProgressMessage()
{
	if (iCommandList.isEmpty())
		iProgress->SetAction(iCommandMessages.at(0));
	else
		iProgress->SetAction(iCommandMessages.at(iCommandList.first()) + "...");
}

void DialogController::DelayedProgressDialogClose()
{
	if (iCommandList.count() == 0)
	{
		iProgress->hide();
		if (!iError->isVisible())
			iParent->SetActiveView();
	}
	else
	{
		SetProgressMessage();
	}
}
