/*
 * PagesView.cpp
 *
 *  Created on: 4.11.2009
 *      Author: thyv
 */

#include "PagesView.h"
#include "WPData.h"
#include "BlogView.h"
#include "wpscrollarea.h"
#include "WPNetworkEngine.h"
#include "textlistitem.h"

#include <QDateTime>
#include <QMenu>
#include <QDesktopWidget>

PagesView::PagesView(BlogView* aParent) : QWidget(aParent), iParent(aParent)
{
	QVBoxLayout* layout = new QVBoxLayout(this);
	iScrollArea = new WPScrollArea;
	iScrollArea->setFocusPolicy(Qt::NoFocus);
	layout->setSpacing(0);
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(iScrollArea);

	iLocalDraftList = new CollectionList(this, iScrollArea);
	iLocalDraftList->DisableMenuAction(CollectionList::EOpen);
	iLocalDraftList->SetTitle(tr("Drafted pages "), true);

	connect(iLocalDraftList->iActionOpen, SIGNAL(triggered()), this, SLOT(OpenLocalDraft()));
	connect(iLocalDraftList->iActionEdit, SIGNAL(triggered()), this, SLOT(EditLocalDraft()));
	connect(iLocalDraftList->iActionDelete, SIGNAL(triggered()), this, SLOT(DeleteLocalDraft()));
	
	iPageList = new CollectionList(this, iScrollArea);
	iPageList->DisableMenuAction(CollectionList::EOpen);
	iPageList->SetTitle(tr("Published pages "));

	connect(iPageList->iActionOpen, SIGNAL(triggered()), this, SLOT(Open()));
	connect(iPageList->iActionEdit, SIGNAL(triggered()), this, SLOT(Edit()));
	connect(iPageList->iActionDelete, SIGNAL(triggered()), this, SLOT(Delete()));
	
	iScrollArea->addWidget(iLocalDraftList);
	iScrollArea->addWidget(iPageList);
}

PagesView::~PagesView()
{
}

void PagesView::UpdatePageData(const WPDataBlog& aBlog)
{
	QList<WPDataPage> draftPages;
	QList<WPDataPage> publishedPages;
	
	iLocalDraftList->ResetList();
	iPageList->ResetList();
	
	const int localDraftCount = aBlog.iLocalDraftPages.count();
	const int pageCount = aBlog.iPages.count();
	
	for(int i = 0; i < localDraftCount; ++i)
	{
		AddPage(aBlog.iLocalDraftPages[i]);
	}
	for(int i = 0; i < pageCount; ++i)
	{
		if (aBlog.iPages[i].IsDraftOrPending())
			draftPages.append(aBlog.iPages[i]);
		else
			publishedPages.append(aBlog.iPages[i]);
	}
	
	const int draftCount = draftPages.count();
	const int publishedCount = publishedPages.count();
	
	for(int i = 0; i < draftCount; ++i)
	{
		AddPage(draftPages[i]);
	}
	for(int i = 0; i < publishedCount; ++i)
	{
		AddPage(publishedPages[i]);
	}
	iLocalDraftList->SetTitle();
	iPageList->SetTitle();
	iScrollArea->AddStretch();
}
	
void PagesView::AddPage(const WPDataPage& aPage)
{
	QStringList list;
	if (aPage.IsLocalDraft())
		list << "Local draft";
	list << WPNetworkEngine::LocalizedTime(aPage.iDateCreated);
	if (aPage.IsDraftOrPending())
		iLocalDraftList->AddCustomItem(aPage.iPageId, aPage.iTitle, aPage.iWpAuthorDisplayName, list);
	else
		iPageList->AddCustomItem(aPage.iPageId, aPage.iTitle, aPage.iWpAuthorDisplayName, list);
}

void PagesView::Open()
{
	emit OpenClicked(iPageList->iCurrentItem);
}

void PagesView::Edit()
{
	emit EditClicked(iPageList->iCurrentItem);
}

void PagesView::Delete()
{
	emit DeleteClicked(iPageList->iCurrentItem);
}

void PagesView::OpenLocalDraft()
{
	emit OpenClicked(iLocalDraftList->iCurrentItem);
}

void PagesView::EditLocalDraft()
{
	emit EditClicked(iLocalDraftList->iCurrentItem);
}

void PagesView::DeleteLocalDraft()
{
	if (WPDataPost::IsLocalDraftId(iLocalDraftList->iCurrentItem->ItemId()))
	{
		// Local draft -> delete locally
		WPDataBlog* blog = iParent->GetNetworkEngine()->GetCurrentBlog();
		blog->RemoveLocalDraftPage(iLocalDraftList->iCurrentItem->ItemId());
		iLocalDraftList->RemoveItem(iLocalDraftList->iCurrentItem);
		setFocus();
	}
	else
	{
		// Network draft -> delete as a normal post
		emit DeleteClicked(iLocalDraftList->iCurrentItem);
	}
}

void PagesView::setFocus()
{
	if (iLocalDraftList && iLocalDraftList->ItemCount() > 0)
		iLocalDraftList->setFocus();
	else if (iPageList && iPageList->ItemCount() > 0)
	    iPageList->setFocus();
	else
	    QWidget::setFocus();
}
