/*
 * SinglePostView.cpp
 *
 *  Created on: 14.12.2009
 *      Author: apoi
 */

#include <QList>
#include <QVBoxLayout>
#include <QAction>
#include <QString>
#include <QDesktopWidget>

#include "AddComment.h"
#include "BlogView.h"
#include "SinglePostData.h"
#include "SinglePostView.h"
#include "WPData.h"
#include "wpcommentlist.h"
#include "wpscrollarea.h"
#include "WPNetworkEngine.h"
#include "WPData.h"

SinglePostView::SinglePostView(BlogView& aParent) : MDragWidget(&aParent), iParent(aParent)
{
#ifdef Q_WS_MAEMO_5
	setWindowFlags(Qt::Window);
	setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
	setWindowTitle(tr("Single post view"));
	SetScreenMaxSize();
	
	iPostId.clear();
	iListbox = NULL;
	QVBoxLayout* layout = new QVBoxLayout(this);

	iScrollArea = new WPScrollArea(this);
	layout->setSpacing(0);
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(iScrollArea);
	iScrollArea->setWidgetResizable(true);
	iScrollArea->setFocusPolicy(Qt::NoFocus);
	iScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	
	SetSoftKeys();
}

SinglePostView::~SinglePostView()
{

}

void SinglePostView::SetScreenMaxSize()
{
	setMaximumWidth(QApplication::desktop()->screenGeometry().size().width());
}

void SinglePostView::resizeEvent(QResizeEvent *e)
{
	SetScreenMaxSize();
	QWidget::resizeEvent(e);
}

void SinglePostView::SetSoftKeys()
{
#ifdef Q_OS_SYMBIAN
	// Prepare softkeys
	QAction* rightSoftKey = new QAction(tr("Back"), this);
	connect(rightSoftKey, SIGNAL(triggered()), this, SLOT(CloseView()));
	rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
	addAction(rightSoftKey);
#else
#ifndef Q_WS_MAEMO_5
	QPushButton * backButton = new QPushButton(tr("Back"));
	connect(backButton, SIGNAL(clicked()), this, SLOT(CloseView()));
	layout()->addWidget(backButton);
#endif
#endif
}

void SinglePostView::ShowPost(const WPDataBlog& aBlog, const WPDataPost& aPost)
{
	// List of comments of this post
	iPostId = aPost.iPostId;
	iPostComments.clear();
	const WPDataComment* comment = 0;
	QListIterator<WPDataComment> i(aBlog.iComments);
	while (i.hasNext())
	{
		comment = &(i.next());
		if (comment->iPostId == iPostId)
		{
			// We assume that comments are sorted by time (newest first)
			iPostComments.prepend(*comment);
		}
	}

	// Construct a list of comment objects
	delete iListbox;
	iListbox = NULL;

	// These are deleted when the layout (in iListBox) gets deleted
	iPostData = new SinglePostData(this, aPost);
	iListbox = new WPCommentList(this, iPostComments, *iParent.GetNetworkEngine(), *iScrollArea, true);
	iScrollArea->setWidget(iPostData);
	iScrollArea->addWidget(iListbox);
	iScrollArea->AddStretch();

	if (!aPost.IsLocalDraft())
	{
		AddComment* newComment = new AddComment(this, aPost.iPostId, iParent);
		iScrollArea->addWidget(newComment);
	}
	iListbox->setFocus();
	ScrollToTop();
}

void SinglePostView::UpdateData(const WPDataBlog& aBlog)
{
	if (!iPostId.isEmpty())
	{
		WPDataPost* post = iParent.GetNetworkEngine()->GetPost(iPostId);
		if (post)
			ShowPost(aBlog, *post);
	}
}

bool SinglePostView::MDragWidgetIsDragging() const
{
	return iScrollArea->IsDragging();
}

QSize SinglePostView::VisibleSize() const
{
	return iScrollArea->GetVisibleContentSize();
}

void SinglePostView::ScrollToTop()
{
	iScrollArea->ScrollToTop();
}

void SinglePostView::CloseView()
{
	iPostId.clear();
	iParent.ShowTabbedView();
}
