/*
 * WPData.h
 *
 *  Created on: 5.11.2009
 *      Author: jkal
 */

#ifndef WPDATA_H_
#define WPDATA_H_

#include <QDate>
#include <QList>
#include <QPair>
#include <QMap>
#include <QDataStream>
#include <QStringList>

class WPDataPage;
class WPDataBlog;
class WPDataPost;
class WPDataCategory;
class WPDataTag;
class WPDataComment;
class WPDataStats;

const int KDataUsersBlogsVersionNumber = 1;
const int KDataBlogVersionNumber  = 2;
const int KDataPostVersionNumber = 1;
const int KDataPageVersionNumber = 1;
const int KDataCategoryVersionNumber = 1;
const int KDataTagVersionNumber = 1;
const int KDataCommentVersionNumber = 1;
const int KDataStatsVersionNumber = 1;

class WPDataUsersBlogs : public QObject
	{
public:
	QList<WPDataBlog> iBlogs;
	int iActiveBlogIndex;
	int iDataUsersBlogsVersionNumber;
	};
	QDataStream &operator<<(QDataStream &out, const WPDataUsersBlogs &dataUsersBlogs );
	QDataStream &operator>>(QDataStream &in, WPDataUsersBlogs &dataUsersBlogs );

   class WPDataStats {
        public:
                QList<QPair<QDate, int> > iDailyVisits;
                int iTotalVisits;
                int iDataStatsVersionNumber;
        };
        QDataStream &operator<<(QDataStream &out, const WPDataStats &dataStats );
        QDataStream &operator>>(QDataStream &in, WPDataStats &dataStats );

class WPDataBlog
	{
public:
	void RemovePage(QString aPageId);
	void RemovePost(QString aPostId);
	void RemoveLocalDraftPage(QString aPageId);
	void RemoveLocalDraftPost(QString aPostId);
	
public:
	QList<WPDataPage> iPages;
	QList<WPDataPage> iLocalDraftPages;
	QList<WPDataPost> iPosts;
	QList<WPDataPost> iLocalDraftPosts;
	QList<WPDataCategory> iCategories;
	QList<WPDataTag> iTags;
	QList<WPDataComment> iComments;

        WPDataStats iStats;

	QMap<QString, QString> iPageStatuses;
	QMap<QString, QString> iPostStatuses;
	QMap<QString, QString> iCommentStatuses;

	QString iUsername;
	QString iPassword;
	QString iUrl;
	QString iRpcUrl;
	QString iBlogName;
	QString iBlogId;
	QString iIsAdmin;
	QString iNumberOfPosts;
        QString iApiKey;
	bool iResizePhotos;
	int iDataBlogVersionNumber;
	};

QDataStream &operator<<(QDataStream &out, const WPDataBlog &dataBlog );
QDataStream &operator>>(QDataStream &in, WPDataBlog &dataBlog );

class WPDataPage
	{
public:
	bool IsLocalDraft() const;
	bool IsDraftOrPending() const;
	static bool IsLocalDraftId(const QString& aPageId) { return aPageId.left(6) == "local_"; }
	
public:
	QString iDateCreated;
	QString iPageStatus;
	QString iDescription;
	QString iTitle;
	QString iLink;
	QString iPermaLink;
	QString iExcerpt;
	QString iTextMore;
	QString iWpSlug;
	QString iWpPassword;
	QString iWpAuthor;
	QString iWpPageParentTitle;
	QString iWpAuthorDisplayName;
	QString iDateCreatedGmt;
	QString iWpPageTemplate;
	QString iUserId;
	QString iPageId;
	QString iAllowComments;
	QString iAllowPings;
	QString iPageParentId;
	QString iWpPageOrder;
	QString iWpAuthorId;
	// TODO: Add support for Categories
	// TODO: Add support for custom fields
	int iDataPageVersionNumber;
	};

QDataStream &operator<<(QDataStream &out, const WPDataPage &dataPage );
QDataStream &operator>>(QDataStream &in, WPDataPage &dataPage );

class WPDataPost
	{
public:
	bool IsLocalDraft() const;
	bool IsDraftOrPending() const;
	static bool IsLocalDraftId(const QString& aPostId) { return aPostId.left(6) == "local_"; }
	
public:
	QString iDateCreated;
	QString iDescription;
	QString iTitle;
	QString iLink;
	QString iPermaLink;
	QString iUserId;
	QString iPostId;
	QString iMtExcerpt;
	QString iMtTextMore;
	QString iMtAllowComments;
	QString iMtAllowPings;
	QString iMtKeywords;
	QString iWpSlug;
	QString iWpPassword;
	QString iWpAuthorId;
	QString iWpAuthorDisplayName;
	QString iDateCreatedGmt;
	QString iPostStatus;
	QStringList iCategories;
	// TODO: Add support for custom fields
	int iDataPostVersionNumber;
	};
QDataStream &operator<<(QDataStream &out, const WPDataPost &dataPost );
QDataStream &operator>>(QDataStream &in, WPDataPost &dataPost );

class WPDataCategory
	{
public:
	QString iCategoryId;
	QString iParentId;
	QString iDescription;
	QString iCategoryDescription;
	QString iCategoryName;
	QString iHtmlUrl;
	QString iRssUrl;
	
	int iDataCategoryVersionNumber;
	};
QDataStream &operator<<(QDataStream &out, const WPDataCategory &dataCategory );
QDataStream &operator>>(QDataStream &in, WPDataCategory &dataCategory );

class WPDataTag
	{
public:
	QString iTagId;
	QString iName;
	QString iCount;
	QString iSlug;
	QString iHtmlUrl;
	QString iRssUrl;
	int iDataTagVersionNumber;
	};
QDataStream &operator<<(QDataStream &out, const WPDataTag &dataTag );
QDataStream &operator>>(QDataStream &in, WPDataTag &dataTag );

class WPDataComment
	{
public:
	QString iDateCreatedGmt;
	QString iUserId;
	QString iCommentId;
	QString iParent;
	QString iStatus;
	QString iContent;
	QString iLink;
	QString iPostId;
	QString iPostTitle;
	QString iAuthor;
	QString iAuthorUrl;
	QString iAuthorEmail;
	QString iAuthorIp;
	QString iType;
	int iDataCommentVersionNumber;
	};
QDataStream &operator<<(QDataStream &out, const WPDataComment &dataComment );
QDataStream &operator>>(QDataStream &in, WPDataComment &dataComment );



class WPDataRsdApi
	{
public:
	QString iName;
	QString iBlogId;
	QString iPreferred;
	QString iApiLink;
	};

#endif /* WPXMLPARSER_H_ */
