/*
 * WPXmlFileUploadParser.cpp
 *
 *  Created on: 5.11.2009
 *      Author: jkal
 */

#include "WPXmlFileUploadParser.h"

WPXmlFileUploadParser::WPXmlFileUploadParser(const QString aResponse, QString& aUrl)
: QXmlStreamReader(aResponse), iUrl(aUrl)
	{
	}

WPXmlFileUploadParser::~WPXmlFileUploadParser()
	{
	}

void WPXmlFileUploadParser::HandleMemberTag()
	{
	QString tagName;
	QString value;
	int depth = 1;
	readNext();
	while (!atEnd() && depth > 0)
		{
		if (isStartElement())
			{
			if (name() == "name")
				tagName = readElementText();
			else if (name() == "string")
				value = readElementText();
			else
				depth++;
			}
		else if (isEndElement())
			depth--;
		readNext();
		}
	
	if (tagName == "url")
		iUrl = value;
	}

void WPXmlFileUploadParser::skipUnknownElement()
	{
	// Reading repeatedly until we get to the same level as we are now.
	int depth = 1;
	while(!atEnd() && depth > 0)
		{
		if(isEndElement())
			depth--;
		else if(isStartElement())
			depth++;
		
		readNext();
		}
	}

void WPXmlFileUploadParser::AddTag(const QString aTag)
	{
	iTagStack.append(aTag);
	}

void WPXmlFileUploadParser::RemoveTag()
	{
	iTagStack.removeLast();
	}

QStringList WPXmlFileUploadParser::TagStack() const
	{
	return iTagStack;
	}

void WPXmlFileUploadParser::ParseResponse()
	{
	while (!atEnd())
		{
		readNext();
		if (isStartElement())
			HandleTag();
		else if (isEndElement())
			RemoveTag();
		}
	}

void WPXmlFileUploadParser::HandleTag()
	{
	QString tagName = name().toString();

	if (tagName == "methodResponse")
		{
		if (TagStack().size() == 0)
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "params")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 1 && tagStack[0] == "methodResponse")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "param")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 2 && tagStack[0] == "methodResponse"
				&& tagStack[1] == "params")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "value")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 3 && tagStack[0] == "methodResponse"
				&& tagStack[1] == "params" && tagStack[2] == "param")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "struct")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 4 && tagStack[0] == "methodResponse" && tagStack[1] == "params"
				&& tagStack[2] == "param" && tagStack[3] == "value")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "member")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 5 && tagStack[0] == "methodResponse" && tagStack[1] == "params"
				&& tagStack[2] == "param" && tagStack[3] == "value" && tagStack[4] == "struct")
			HandleMemberTag();
		else
			skipUnknownElement();
		}
	}
