/*
 * WPXmlParser.cpp
 *
 *  Created on: 5.11.2009
 *      Author: jkal
 */

#include "WPXmlParser.h"

WPXmlParser::WPXmlParser(const QString aResponse)
: QXmlStreamReader(aResponse), iFaultCode(0)
{
}

WPXmlParser::~WPXmlParser()
{
}

void WPXmlParser::skipUnknownElement()
{
	// Reading repeatedly until we get to the same level as we are now.
	int depth = 1;
	while(!atEnd() && depth > 0)
    {
		if(isEndElement())
			depth--;
		else if(isStartElement())
			depth++;
		
		readNext();
    }
}

void WPXmlParser::ParseResponse()
{
    readNextStartElement();
	while (!atEnd())
    {
        if (name() == "methodResponse")
            HandleMethodResponseTag();
        else
            readNextStartElement();
    }
}

void WPXmlParser::HandleMethodResponseTag()
{
    while (readNextStartElement())
    {
        if (name() == "params")
            HandleParamsTag();
        else if (name() == "fault")
            HandleFaultTag();
    }
}

void WPXmlParser::HandleParamsTag()
{
    while (readNextStartElement())
    {
        if (name() == "param")
            HandleParamTag();
    }
}

void WPXmlParser::HandleParamTag()
{
    while (readNextStartElement())
    {
        if (name() == "value")
            HandleFirstValueTag();
    }
}

void WPXmlParser::HandleFirstValueTag()
{
    while (readNextStartElement())
    {
        if (name() == "array")
            HandleArrayTag();
    }
}

void WPXmlParser::HandleArrayTag()
{
    while (readNextStartElement())
    {
        if (name() == "data")
            HandleDataTag();
    }
}

void WPXmlParser::HandleDataTag()
{
    while (readNextStartElement())
    {
        if (name() == "value")
            HandleSecondValueTag();
    }
}

void WPXmlParser::HandleSecondValueTag()
{
    while (readNextStartElement())
    {
        if (name() == "struct")
        {
            CreateNewItem();
            HandleStructTag();
        }
    }
}

void WPXmlParser::HandleStructTag()
{
    while (readNextStartElement())
    {
        if (name() == "member")
            HandleMemberTag();
    }
}

void WPXmlParser::HandleFaultTag()
{
    while (readNextStartElement())
    {
        if (name() == "value")
            HandleFaultValueTag();
    }
}

void WPXmlParser::HandleFaultValueTag()
{
    while (readNextStartElement())
    {
        if (name() == "struct")
            HandleFaultStructTag();
    }
}

void WPXmlParser::HandleFaultStructTag()
{
    while (readNextStartElement())
    {
        if (name() == "member")
            HandleFaultMemberTag();
    }
}

void WPXmlParser::HandleFaultMemberTag()
{
	QString tagName;
	QString value;
	int depth = 1;
	readNext();
	while (!atEnd() && depth > 0)
		{
		if (isStartElement())
			{
			if (name() == "name")
				tagName = readElementText();
			else if (name() == "int" || name() == "string")
				value = readElementText();
			else
				depth++;
			}
		else if (isEndElement())
			depth--;
		readNext();
		}
	
	if (tagName == "faultCode")
	{
		bool success = false;
		iFaultCode = value.toInt(&success, 10);
		if (!success)
			iFaultCode = 0; // TODO: Error handling
	}
}

int WPXmlParser::FaultCode() const
	{
	return iFaultCode;
	}
