/*
 * WPXmlPostsParser.cpp
 *
 *  Created on: 5.11.2009
 *      Author: jkal
 */

#include "WPXmlPostsParser.h"
#include "WPData.h"

WPXmlPostsParser::WPXmlPostsParser(const QString aResponse, WPDataBlog& aData)
: WPXmlParser(aResponse), iData(aData)
	{
	}

WPXmlPostsParser::~WPXmlPostsParser()
	{
	}

void WPXmlPostsParser::HandleMemberTag()
	{
	const int dataSize = iData.iPosts.size();
	if (dataSize == 0)
		return; // Should not be called like this but let's be sure
	WPDataPost& data = iData.iPosts[dataSize - 1];
	QString tagName;
	QString value;
	int depth = 1;
	while (!atEnd() && depth > 0)
		{
		readNext();
		if (isStartElement())
			{
			if (name() == "name")
				tagName = readElementText();
			else if (name() == "int" || name() == "boolean" || name() == "string" || name() == "dateTime.iso8601")
				value = readElementText();
			else if (name() == "value" && tagName == "categories")
				HandleCategoryArray(data);
			else
				depth++;
			// TODO: Handle custom fields
			}
		else if (isEndElement())
			depth--;
		}
	
	if (tagName == "dateCreated")
		data.iDateCreated = value;
	else if (tagName == "description")
		data.iDescription = value;
	else if (tagName == "title")
		data.iTitle = value;
	else if (tagName == "link")
		data.iLink = value;
	else if (tagName == "permaLink")
		data.iPermaLink = value;
	else if (tagName == "userid")
		data.iUserId = value;
	else if (tagName == "postid")
		data.iPostId = value;
	else if (tagName == "mt_excerpt")
		data.iMtExcerpt = value;
	else if (tagName == "mt_text_more")
		data.iMtTextMore = value;
	else if (tagName == "mt_allow_comments")
		data.iMtAllowComments = value;
	else if (tagName == "mt_allow_pings")
		data.iMtAllowPings = value;
	else if (tagName == "mt_keywords")
		data.iMtKeywords = value;
	else if (tagName == "wp_slug")
		data.iWpSlug = value;
	else if (tagName == "wp_password")
		data.iWpPassword = value;
	else if (tagName == "wp_author_id")
		data.iWpAuthorId = value;
	else if (tagName == "wp_author_display_name")
		data.iWpAuthorDisplayName = value;
	else if (tagName == "date_created_gmt")
		data.iDateCreatedGmt = value;
	else if (tagName == "post_status")
		data.iPostStatus = value;
	}

void WPXmlPostsParser::CreateNewItem()
	{
	WPDataPost newItem;
	iData.iPosts.append(newItem);
	}

void WPXmlPostsParser::HandleCategoryArray(WPDataPost& aData)
	{
	int depth = 1;
	while (!atEnd() && depth > 0)
		{
		readNext();
		if (isStartElement())
			{
			if (name() == "string")
				aData.iCategories.append(readElementText());
			else
				depth++;
			}
		else if (isEndElement())
			depth--;
		}
	}
