/*
 * WPXmlResponseParser.cpp
 *
 *  Created on: 13.11.2009
 *      Author: jkal
 */

#include <QDomDocument>
#include <QString>

#include "WPXmlResponseParser.h"

WPXmlResponseParser::WPXmlResponseParser()
	{
	}

bool WPXmlResponseParser::ParseResponse(const QString& aContent)
	{
	QString errorStr;
	int errorLine;
	int errorColumn;
	
	QDomDocument doc;
	const int error = doc.setContent(aContent, &errorStr, &errorLine, &errorColumn);
	if (!error)
		{
        return false;
		}
	
	QDomElement root = doc.documentElement();
	if (root.tagName() != "methodResponse")
		{
		return false;
		}
	
	return ParseMethodResponseElement(root);
	}

WPXmlResponseParser::~WPXmlResponseParser()
	{
	}

bool WPXmlResponseParser::ParseMethodResponseElement(const QDomElement& aElement)
	{
	QDomNode child = aElement.firstChild();
	while (!child.isNull())
		{
		if (child.toElement().tagName() == "params")
			return ParseParamsElement(child.toElement());
		child = child.nextSibling();
		}
	return false;
	}

bool WPXmlResponseParser::ParseParamsElement(const QDomElement& aElement)
	{
	QDomNode child = aElement.firstChild();
	while (!child.isNull())
		{
		if (child.toElement().tagName() == "param")
			return ParseParamElement(child.toElement());
		child = child.nextSibling();
		}
	return false;
	}

bool WPXmlResponseParser::ParseParamElement(const QDomElement& aElement)
	{
	QDomNode child = aElement.firstChild();
	while (!child.isNull())
		{
		if (child.toElement().tagName() == "value")
			return ParseValueElement(child.toElement());
		child = child.nextSibling();
		}
	return false;
	}

bool WPXmlResponseParser::ParseValueElement(const QDomElement& aElement)
	{
	QDomNode child = aElement.firstChild();
	while (!child.isNull())
		{
		if (child.toElement().tagName() == "boolean")
			return ParseBooleanElement(child.toElement());
		else if (child.toElement().tagName() == "string")
			return ParseStringElement(child.toElement());
		child = child.nextSibling();
		}
	return false;
	}

bool WPXmlResponseParser::ParseBooleanElement(const QDomElement& aElement)
	{
	return aElement.text() == "1";
	}

bool WPXmlResponseParser::ParseStringElement(const QDomElement& aElement)
{
	// Parses newPage int response (pageId)
	// Just try to convert, since all page id int values should be acceptable
	bool success;
	aElement.text().toInt(&success);
	return success;
}
