/*
 * WPXmlStatusesParser.cpp
 *
 *  Created on: 5.11.2009
 *      Author: jkal
 */

#include "WPXmlStatusesParser.h"
#include "WPData.h"

WPXmlStatusesParser::WPXmlStatusesParser(const QString aResponse, QMap<QString, QString>& aData)
: QXmlStreamReader(aResponse), iData(aData)
	{
	}

WPXmlStatusesParser::~WPXmlStatusesParser()
	{
	}

void WPXmlStatusesParser::HandleMemberTag()
	{
	QString key;
	QString value;
	int depth = 1;
	readNext();
	while (!atEnd() && depth > 0)
		{
		if (isStartElement())
			{
			if (name() == "name")
				key = readElementText();
			else if (name() == "string")
				value = readElementText();
			else
				depth++;
			}
		else if (isEndElement())
			depth--;
		readNext();
		}
	iData.insert(key, value);
	}

void WPXmlStatusesParser::skipUnknownElement()
	{
	// Reading repeatedly until we get to the same level as we are now.
	int depth = 1;
	while(!atEnd() && depth > 0)
		{
		if(isEndElement())
			depth--;
		else if(isStartElement())
			depth++;
		
		readNext();
		}
	}

void WPXmlStatusesParser::AddTag(const QString aTag)
	{
	iTagStack.append(aTag);
	}

void WPXmlStatusesParser::RemoveTag()
	{
	iTagStack.removeLast();
	}

QStringList WPXmlStatusesParser::TagStack() const
	{
	return iTagStack;
	}

void WPXmlStatusesParser::ParseResponse()
	{
	while (!atEnd())
		{
		readNext();
		if (isStartElement())
			HandleTag();
		else if (isEndElement())
			RemoveTag();
		}
	}

void WPXmlStatusesParser::HandleTag()
	{
	QString tagName = name().toString();

	if (tagName == "methodResponse")
		{
		if (TagStack().size() == 0)
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "params")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 1 && tagStack[0] == "methodResponse")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "param")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 2 && tagStack[0] == "methodResponse"
				&& tagStack[1] == "params")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "value")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() >= 3 && tagStack[0] == "methodResponse"
				&& tagStack[1] == "params" && tagStack[2] == "param")
			{
			if (tagStack.size() == 3 || (tagStack.size() == 6 &&
					tagStack[3] == "value" && tagStack[4] == "array" &&
					tagStack[5] == "data"))
				AddTag(tagName);
			else
				skipUnknownElement();
			}
		else
			skipUnknownElement();
		}
	else if (tagName == "array")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 4 && tagStack[0] == "methodResponse"
				&& tagStack[1] == "params" && tagStack[2] == "param"
				&& tagStack[3] == "value")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "data")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 5 && tagStack[0] == "methodResponse"
				&& tagStack[1] == "params" && tagStack[2] == "param"
				&& tagStack[3] == "value" && tagStack[4] == "array")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "struct")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 4 && tagStack[0] == "methodResponse" && tagStack[1] == "params"
				&& tagStack[2] == "param" && tagStack[3] == "value")
			AddTag(tagName);
		else
			skipUnknownElement();
		}
	else if (tagName == "member")
		{
		QStringList tagStack = TagStack();
		if (tagStack.size() == 5 && tagStack[0] == "methodResponse" && tagStack[1] == "params"
				&& tagStack[2] == "param" && tagStack[3] == "value" && tagStack[4] == "struct")
			HandleMemberTag();
		else
			skipUnknownElement();
		}
	}
