
#include <QWidget>
#include <QDateTime>

#include <iostream>

#include "WPNetworkEngine.h"
#include "addpostview.h"
#include "BlogView.h"
#include "WPData.h"

AddPostView::AddPostView(BlogView *parent) : QFrame(parent), iParent(parent)
{
#ifdef Q_WS_MAEMO_5
	setWindowFlags(Qt::Window);
	setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
	ui.setupUi(this);
	ui.scrollArea->setWidget(ui.layoutWidget);
	ui.layoutWidget->setContentsMargins(5, 5, 5, 5);
	
	iCategoryListLength = 30; // TODO some better method

	ui.titleEdit->SetExplanation(tr("Post title"));
	ui.passwordEdit->SetExplanation(tr("Password"));
	ui.passwordEdit->SetPasswordMasking(true);
	ui.tagsEdit->SetExplanation(tr("Tags"));
	ui.contentsEdit->setExplanationText(tr("Tap to begin writing"));
	
#ifdef Q_OS_SYMBIAN
	// Prepare softkeys
	QAction* leftSoftKey = new QAction(tr("Save"), this);
	connect(leftSoftKey, SIGNAL(triggered()), this, SLOT(SavePost()));
	leftSoftKey->setSoftKeyRole(QAction::PositiveSoftKey);
	addAction(leftSoftKey);
	QAction* rightSoftKey = new QAction(tr("Cancel"), this);
	connect(rightSoftKey, SIGNAL(triggered()), iParent, SLOT(ShowTabbedView()));
	rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
	addAction(rightSoftKey);
#else
	QPushButton * saveButton = new QPushButton(tr("Save"));
	saveButton->setObjectName(QString::fromUtf8("Maemo5NavigationButton"));
	connect(saveButton, SIGNAL(clicked()), this, SLOT(SavePost()));
	QHBoxLayout * buttonLayout = new QHBoxLayout;
	buttonLayout->addWidget(saveButton);
#ifndef Q_WS_MAEMO_5
	QPushButton * cancelButton = new QPushButton(tr("Cancel"));
	cancelButton->setObjectName(QString::fromUtf8("Maemo5NavigationButton"));
	connect(cancelButton, SIGNAL(clicked()), iParent, SLOT(ShowTabbedView()));
	buttonLayout->addWidget(cancelButton);
#endif
	ui.verticalLayout->addLayout(buttonLayout);
#endif
	
	connect(ui.addPhotosButton, SIGNAL(clicked(bool)), this, SLOT(AddPhotosClicked(bool)));
	connect(ui.categoriesEdit, SIGNAL(clicked()), this, SLOT(CategoriesClicked()));
	
	ui.scrollArea->DisableScrollAtWidget(ui.statusEdit->view());
}

AddPostView::~AddPostView()
{
}

void AddPostView::NewPost(WPDataBlog *aBlog)
{
	iCategories.clear();
	iDataBlog = aBlog;
	iPostId = "";
	iAddedImages.clear();
	ui.titleEdit->setText("");
        ui.contentsEdit->reset();
        ui.contentsEdit->setText("");
	ui.tagsEdit->setText("");
	ui.dateTimeEdit->setDateTime(QDateTime::currentDateTime().toLocalTime());
	ui.passwordEdit->setText("");
	ui.addPhotosButton->setText(tr("Add photos") + "...");
	ui.categoriesEdit->setText(tr("Add categories") + "...");
	
	// Statuses
	ui.statusEdit->clear();
	QStringList keyList, valueList;
	QMap<QString, QString>::const_iterator i;
	for (i = aBlog->iPostStatuses.constBegin(); i != aBlog->iPostStatuses.constEnd(); i++)
	{
		keyList.append(i.key());
		valueList.append(i.value());
	}
	ui.statusEdit->addItems(valueList);
	ui.statusEdit->addItem("Local draft");
	ui.titleEdit->setFocus();
}

void AddPostView::EditPost(WPDataBlog *aBlog, const WPDataPost *aPost)
{
        iDataBlog = aBlog;
	iPostId = aPost->iPostId;
	iAddedImages.clear();
	ui.titleEdit->setText(aPost->iTitle);
        ui.contentsEdit->reset();
	ui.contentsEdit->setText(aPost->iDescription);
	ui.tagsEdit->setText(aPost->iMtKeywords);
	ui.dateTimeEdit->setDateTime(QDateTime::fromString(aPost->iDateCreated, "yyyyMMddTHH:mm:ss"));
	ui.passwordEdit->setText(aPost->iWpPassword);
	ui.addPhotosButton->setText(tr("Add photos") + "...");

	// Collect categories
	iCategories = aPost->iCategories;
	ShowSelectedCategories();
	
	// Fill the status box, preselect the correct item
	ui.statusEdit->clear();
	QStringList keyList, valueList;
	QMap<QString, QString>::const_iterator j;
	for (j = aBlog->iPostStatuses.constBegin(); j != aBlog->iPostStatuses.constEnd(); j++)
	{
		keyList.append(j.key());
		valueList.append(j.value());
	}
	ui.statusEdit->addItems(valueList);
	if (WPDataPost::IsLocalDraftId(iPostId))
	{
		ui.statusEdit->addItem("Local draft");
		ui.statusEdit->setCurrentIndex(ui.statusEdit->count() - 1);
	}
	// Using regular for loop as we need to get the index
	for (int i = 0; i < keyList.count(); ++i)
	{
		if (aPost->iPostStatus.compare(keyList.at(i)) == 0)
		{
			ui.statusEdit->setCurrentIndex(i);
			break;
		}
	}
	ui.titleEdit->setFocus();
}

void AddPostView::SavePost()
{
	if (ui.statusEdit->currentText() == "Local draft")
	{
		if (iAddedImages.count() > 0)
		{
			emit ShowErrorNote(tr("Images are not saved in local drafts. Please re-add images when publishing the post."));
		}

		if (iPostId.isEmpty())
		{
			WPDataPost post;
			QString id = "local_";
			id += QString::number(qrand());
			post.iPostId = id;
			post.iTitle = ui.titleEdit->text();
			post.iCategories = iCategories;
			post.iDescription = ui.contentsEdit->text();
			post.iMtKeywords = ui.tagsEdit->text();
			post.iDateCreated = WPNetworkEngine::ISO8601Time(ui.dateTimeEdit->dateTime());
			post.iWpPassword = ui.passwordEdit->text();
			iDataBlog->iLocalDraftPosts.append(post);
		}
		else
		{
			WPDataPost* post = iParent->GetNetworkEngine()->GetLocalDraftPost(iPostId);
			if (post)
			{
				post->iTitle = ui.titleEdit->text();
				post->iCategories = iCategories;
				post->iDescription = ui.contentsEdit->text();
				post->iMtKeywords = ui.tagsEdit->text();
				post->iDateCreated = WPNetworkEngine::ISO8601Time(ui.dateTimeEdit->dateTime());
				post->iWpPassword = ui.passwordEdit->text();
			}
		}
		emit UpdatePostData(*iDataBlog);
	}
	else
	{
		QMap<QString, QString> content;
		content.insert("title", ui.titleEdit->text());
		content.insert("description", ui.contentsEdit->text());
		content.insert("post_status", iDataBlog->iPostStatuses.key(ui.statusEdit->currentText()));
		content.insert("mt_keywords", ui.tagsEdit->text());
		content.insert("dateCreated", WPNetworkEngine::ISO8601Time(ui.dateTimeEdit->dateTime().toUTC()));
		content.insert("wp_password", ui.passwordEdit->text());
		
		QMap<QString, QStringList> listContent;
		listContent.insert("categories", iCategories);
		
		if (iPostId.isEmpty() || WPDataPost::IsLocalDraftId(iPostId))
			emit NewPost(content, listContent, "1", iAddedImages, iPostId);
		else
			emit EditPost(iPostId, content, listContent, "0", iAddedImages);
	}
	iParent->ShowTabbedView();
#ifdef Q_WS_MAEMO_5
	close();
#endif
}

void AddPostView::ImagesAdded(const QStringList& aFilenames)
{
	iAddedImages = aFilenames;
	
	QString labelText;
	if (iAddedImages.size() == 0)
		labelText = tr("Add photos") + "...";
	else if (iAddedImages.size() == 1) 
		labelText = tr("Adding") + " 1 " + tr("new photo"); 
	else
		labelText = tr("Adding") + " " + QString::number(iAddedImages.size()) + " " + tr("new photos");
	
	ui.addPhotosButton->setText(labelText);
}


void AddPostView::SelectCategories(const QStringList* aCategories)
{
	iCategories.clear();
	foreach (QString str, *aCategories)
		iCategories.append(str);
	ShowSelectedCategories();
}

void AddPostView::ShowSelectedCategories()
{
	QString textList = "";
	foreach (QString str, iCategories)
		textList += str + ", ";
	if (textList.length() > 0)
		textList = textList.mid(0, textList.length() - 2);
	else
		textList = tr("Add categories") + "...";

	if (textList.length() > iCategoryListLength)
		textList = textList.left(iCategoryListLength) + "...";
	ui.categoriesEdit->setText(textList);
}

void AddPostView::CategoriesClicked()
{
	iParent->ShowCategoryView(&iCategories);
}

void AddPostView::AddPhotosClicked(bool /*aDummy*/)
{
	iParent->ShowImageList(iAddedImages, true);
}

void AddPostView::setFocus()
{
	ui.scrollArea->ScrollToTop();
	ui.scrollArea->setFocus();
}
