
#include <QList>
#include <QBoxLayout>
#include <QDesktopWidget>
#include <QRect>

#include "commentsview.h"
#include "BlogView.h"
#include "WPData.h"
#include "wpcommentlist.h"
#include "wpscrollarea.h"
#include "WPHomeScreenInfo.h"
#include "WPLogger.h"

CommentsView::CommentsView(BlogView& aParent) : QFrame(&aParent), iParent(aParent)
{
	iListbox = NULL;
	iHomeInfo = NULL;
	
	QVBoxLayout* layout = new QVBoxLayout(this);
	layout->setSpacing(0);
	layout->setContentsMargins(0, 0, 0, 0);
	
	iScrollArea = new WPScrollArea(this);
	iScrollArea->setFocusPolicy(Qt::NoFocus);
	// Set scrollbar to visible at all times to eliminate redrawing when
	// the scrollbar is added. After all, there should be not so many blog
	// that don't have at least three comments.
	iScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
	layout->addWidget(iScrollArea);
}

CommentsView::~CommentsView()
{

}

void CommentsView::resizeEvent(QResizeEvent *e)
{
	// Set maximum width for the contained items, this
	// will reduce the amount of redraws
	int availableWidth = iScrollArea->GetVisibleContentSize().width();
	if (iListbox)
	    iListbox->setMaximumWidth(availableWidth);
	if (iHomeInfo)
	    iHomeInfo->setMaximumWidth(availableWidth);
	QFrame::resizeEvent(e);
}

void CommentsView::UpdatePostData(const WPDataBlog& aBlog)
{
    if (iHomeInfo)
        iHomeInfo->UpdateData(&aBlog);
}

void CommentsView::UpdateCommentsData(const WPDataBlog& aBlog)
{
	WPLogger::Instance().LogRow("CommentsView::UpdateCommentsData called.");
	delete iListbox;
	iListbox = NULL;
	iNewComments.clear();
	
	WPLogger::Instance().LogRow("Data reset.");

	// iHomeInfo is deleted by the layout where it resides.
	// Layout is in iListbox, so we can allocate a new HomeScreenInfo
	// from the heap without leaking memory.
	iHomeInfo = new WPHomeScreenInfo(*this);
	iHomeInfo->UpdateData(&aBlog);

	WPLogger::Instance().LogRow("HomeInfo created.");
	
	const int commentCount = aBlog.iComments.size();
	int newCommentCount = 0;
	for (int i = 0; i < commentCount; i++)
		if (aBlog.iComments[i].iStatus == "hold")
			newCommentCount++;
	
#ifdef Q_WS_MAEMO_5
	if (newCommentCount > 0)
		iParent.TabView()->setTabText(0, tr("  Comments") + " (" + QString::number(newCommentCount) + ")  ");
	else
		iParent.TabView()->setTabText(0, tr("  Comments  "));
#else
	if (newCommentCount > 0)
		iParent.TabView()->setTabText(0, tr("Comments") + " (" + QString::number(newCommentCount) + ")");
	else
		iParent.TabView()->setTabText(0, tr("Comments"));
#endif

	WPLogger::Instance().LogRow("Starting to create listbox.");

	iListbox = new WPCommentList(this, aBlog.iComments, *iParent.GetNetworkEngine(), *iScrollArea);
	connect(iListbox, SIGNAL(ReplyToComment(const WPDataComment&)), &iParent, SLOT(ReplyToComment(const WPDataComment&)));
	
	WPLogger::Instance().LogRow("Listbox created.");
	
	iScrollArea->setWidget(iHomeInfo);
	WPLogger::Instance().LogRow("HomeInfo added to scroll area.");
	iScrollArea->addWidget(iListbox);
	WPLogger::Instance().LogRow("CommentsView::UpdateCommentsData exiting.");

	iParent.SetActiveView();
}

void CommentsView::setFocus()
{
	// Set focus to the home info. If the focus was in the
	// first comment, the screen would be scrolled in small
	// screens.
	if (iHomeInfo)
		iHomeInfo->setFocus();
}

QSize CommentsView::VisibleSize() const
{
	return iScrollArea->GetVisibleContentSize();
}
