/*
 * ExplanatoryTextEdit.cpp
 *
 *  Created on: 12.1.2010
 *      Author: apoi
 */

#include "explanatorytextedit.h"

ExplanatoryTextEdit::ExplanatoryTextEdit(QWidget *aParent, const QString& aExplanation)
	: QTextEdit(aParent), iExplanation(aExplanation)
{
        connect(this, SIGNAL(textChanged()), this, SLOT(textChanged()));
	iDisregardNextTextChange = false;
	setText("");

	// trac ticket #23
	document()->setDefaultStyleSheet("a {color: blue;}");
}

ExplanatoryTextEdit::~ExplanatoryTextEdit()
{
}

void ExplanatoryTextEdit::setText(QString aValue)
{
	iValue = aValue;
	iDisregardNextTextChange = true;
	if (!aValue.isEmpty()) {
		QTextEdit::setText(iValue);
	} else {
		QTextEdit::setText(iExplanation);
	}
}

void ExplanatoryTextEdit::setPlainText(QString aValue)
{
	iValue = aValue;
	iDisregardNextTextChange = true;
	if (!aValue.isEmpty()) {
		QTextEdit::setPlainText(iValue);
	} else {
		QTextEdit::setPlainText(iExplanation);
	}
}

QString ExplanatoryTextEdit::text() const
{
	return iValue;
}

void ExplanatoryTextEdit::SetExplanation(const QString& aExplanation)
{
	iExplanation = aExplanation;
	if (iValue.isEmpty())
	{
		iDisregardNextTextChange = true;
		QTextEdit::setPlainText(iExplanation);
	}
}

void ExplanatoryTextEdit::focusInEvent(QFocusEvent *e)
{
	iDisregardNextTextChange = true;
	QTextEdit::setPlainText(iValue);
        QTextEdit::moveCursor(QTextCursor::End);
	QTextEdit::focusInEvent(e);
}

void ExplanatoryTextEdit::focusOutEvent(QFocusEvent *e)
{
	setText(QTextEdit::toPlainText());
	QTextEdit::focusOutEvent(e);
}

void ExplanatoryTextEdit::textChanged()
{
	// Set new value only if the value is set by user
	if (!iDisregardNextTextChange)
		iValue = QTextEdit::toPlainText();
	else
		iDisregardNextTextChange = false;
}

void ExplanatoryTextEdit::insertText(const QString& text) {
     setPlainText(iValue.append(text));
     QTextEdit::moveCursor(QTextCursor::End);
}
