/****************************************************************************
**
** Trolltech hereby grants a license to use the Qt/Eclipse Integration
** plug-in (the software contained herein), in binary form, solely for the
** purpose of creating code to be used with Trolltech's Qt software.
**
** Qt Designer is licensed under the terms of the GNU General Public
** License versions 2.0 and 3.0 ("GPL License"). Trolltech offers users the
** right to use certain no GPL licensed software under the terms of its GPL
** Exception version 1.2 (http://trolltech.com/products/qt/gplexception).
**
** THIS SOFTWARE IS PROVIDED BY TROLLTECH AND ITS CONTRIBUTORS (IF ANY) "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
** TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
** PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
** OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
** EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
** PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** Since we now have the GPL exception I think that the "special exception
** is no longer needed. The license text proposed above (other than the
** special exception portion of it) is the BSD license and we have added
** the BSD license as a permissible license under the exception.
**
****************************************************************************/

#include "BlogView.h"

//#include <QSplashScreen>

#ifdef Q_OS_SYMBIAN
#include <CoeMain.h>
#include "sym_iap_util.h"
static bool NeedsIAPHack();
#endif

#include <QtGui>
#include <QApplication>
void loadStyleSheet(const QString &sheetName);

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
#ifdef Q_OS_SYMBIAN
    a.setNavigationMode(Qt::NavigationModeKeypadDirectional);
    loadStyleSheet("symbian");
    if (NeedsIAPHack())
        qt_SetDefaultIap();
#else
    loadStyleSheet("maemo");
#endif
    
    /*QPixmap pixmap(":/images/blue-l.png");
    QSplashScreen splash(pixmap,Qt::WindowStaysOnTopHint);
	splash.show();
	splash.showMessage("Wait...");
	qApp->processEvents();*/
    
    BlogView view;
#ifdef Q_OS_SYMBIAN
    view.showMaximized();
#else
    view.show();
#endif

    //splash.finish(&view);
    return a.exec();
}


void loadStyleSheet(const QString &sheetName)
{
    QFile file(":/qss/" + sheetName.toLower() + ".qss");
    if(!file.open(QFile::ReadOnly))
        {
        qDebug() << "unable to load style sheet";
        return;
        }
    QString styleSheet = QLatin1String(file.readAll());
    qApp->setStyleSheet(styleSheet);
}

#ifdef Q_OS_SYMBIAN
bool NeedsIAPHack()
{
    if (!CCoeEnv::Static())
        return false;
    TFindFile fileFinder(CCoeEnv::Static()->FsSession());
    CDir* result = NULL;
    _LIT(KS60ProductIDFile, "Series60v*.sis");
    _LIT(KROMInstallDir, "z:\\system\\install\\");
    if (fileFinder.FindWildByDir(KS60ProductIDFile, KROMInstallDir, result))
        return false;
   
    // Sort the file names so that the newest platforms are first
    if (result->Sort(ESortByName | EDescending) || result->Count() == 0)
    {
        delete result;
        return false;
    }
   
    // Parse the version numbers from the file name (for example, Series60v3.1.sis)
    const int major = (*result)[0].iName[9] - '0';
    const int minor = (*result)[0].iName[11] - '0';
    delete result;
    return (major == 3 && minor == 1);
}
#endif
