/*
 * setupblog.cpp
 *
 *  Created on: ?
 *      Author: ?
 */

#include <QAction>
#include <QList>
#include <QMenu>
#include <QPushButton>
#include <QDesktopWidget> 

#include "setupblog.h"
#include "BlogView.h"

SetupBlog::SetupBlog(BlogView *aBlogView, QWidget *parent) : QFrame(parent), iBlogView(aBlogView)
{
#ifdef Q_WS_MAEMO_5
	setWindowFlags(Qt::Window);
	setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
	ui.setupUi(this);
	ui.scrollArea->setWidget(ui.layoutWidget);
	ui.layoutWidget->setContentsMargins(5, 5, 5, 5);
	
	iEditingBlog = false;
	iCancelSet = false;

	iSvg = new QSvgWidget(":/images/wp-logo.svg", this);
	iSvg->setFixedSize(120, 120);

	setTabOrder(iSvg, ui.blogUrl);
	setTabOrder(ui.blogUrl, ui.username);
	setTabOrder(ui.username, ui.password);
	setTabOrder(ui.password, ui.recentPosts);
	setTabOrder(ui.recentPosts, ui.resizePhotos);
	setTabOrder(ui.resizePhotos, ui.resizeLabel);
	
	ui.horizontalLayout->addStretch();
	ui.horizontalLayout->addWidget(iSvg);
	ui.horizontalLayout->addStretch();
	
	ui.blogUrl->SetExplanation("Blog URL");
	ui.username->SetExplanation("Username");
	ui.password->SetExplanation("Password");
	ui.password->SetPasswordMasking(true);
	
	QAction* lsk = new QAction(tr("&Save"), this);

#ifdef Q_OS_SYMBIAN
	lsk->setSoftKeyRole(QAction::PositiveSoftKey);
#else
	QPushButton * saveButton = new QPushButton(tr("&Save"));
	saveButton->setObjectName("Maemo5NavigationButton");
	connect(saveButton, SIGNAL(clicked()), this, SLOT(Save()));
	ui.verticalLayout->addWidget(saveButton);
	lsk->setShortcuts(QKeySequence::Save);
#endif

	addAction(lsk);
	connect(lsk, SIGNAL(triggered()), this, SLOT(Save()));
	
	ui.scrollArea->DisableScrollAtWidget(ui.recentPosts->view());
}

SetupBlog::~SetupBlog()
{

}

void SetupBlog::NewBlog()
{
	SetCancel();
	iEditingBlog = false;
	ui.blogUrl->setText("");
	ui.blogUrl->setReadOnly(false);
	ui.username->setText("");
	ui.password->setText("");
	ui.recentPosts->setCurrentIndex(0);
	ui.resizePhotos->setChecked(true);
}

void SetupBlog::EditBlog(WPDataBlog* aBlog)
{
	SetCancel();
	iEditingBlog = true;
	iEditedBlog = aBlog;
	ui.blogUrl->setText(aBlog->iUrl);
	ui.blogUrl->setReadOnly(true);
	ui.username->setText(aBlog->iUsername);

	ui.password->setText(aBlog->iPassword);
	ui.recentPosts->setCurrentIndex((aBlog->iNumberOfPosts.toInt() / 10 - 1));
	ui.resizePhotos->setChecked(aBlog->iResizePhotos);
}

void SetupBlog::Save()
{
	// TODO what happens if a new blog is added?
	// Should Save/cancel softkeys be disabled, perhaps replaced by "Exit"?
	if (iEditingBlog)
	{
		iEditedBlog->iUsername = ui.username->text();
		iEditedBlog->iPassword = ui.password->text();
		iEditedBlog->iNumberOfPosts = QString::number((ui.recentPosts->currentIndex() + 1) * 10);
		iEditedBlog->iResizePhotos = ui.resizePhotos->isChecked();
		iBlogView->ShowBlogListView();

	}
	else
	{
		iBlogView->SaveBlogSetup(
			ui.blogUrl->text(),
			ui.username->text(),
			ui.password->text(),
			QString::number((ui.recentPosts->currentIndex() + 1) * 10),
			ui.resizePhotos->isChecked()
		);
	}
	#ifdef Q_WS_MAEMO_5
	close();
	#endif
}

void SetupBlog::SetCancel()
{
    if(iCancelSet)
        return;
	// Make sure the negative soft key is Cancel, not Exit.
	// Exit is no more required option when this function is called.
#ifdef Q_OS_SYMBIAN
	QAction* rightSoftKey = new QAction(tr("Cancel"), this);
	connect(rightSoftKey, SIGNAL(triggered()), iBlogView, SLOT(ShowBlogListView()));
	rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
	addAction(rightSoftKey);
#else

#ifndef Q_WS_MAEMO_5
	QPushButton * cancelButton = new QPushButton(tr("&Cancel"));
	ui.verticalLayout->addWidget(cancelButton);
	connect(cancelButton, SIGNAL(clicked()), iBlogView, SLOT(ShowBlogListView()));

#endif
#endif
	iCancelSet = true;

}

void SetupBlog::SetLogoVisibility()
{
	QSize window = QApplication::desktop()->screenGeometry().size();
	if (window.height() < 640)
	{
		iSvg->hide();
	}
	else
	{
		iSvg->show();
	}	
}

void SetupBlog::resizeEvent(QResizeEvent *e)
{
	SetLogoVisibility();
	QFrame::resizeEvent(e);
}

void SetupBlog::setFocus()
{
	ui.scrollArea->ScrollToTop();
	ui.scrollArea->setFocus();
	SetLogoVisibility();

	if (!iEditingBlog)
	{
		// Default title (set in BlogView) is "Edit blog", so let's change that
		iBlogView->setWindowTitle("Add new blog");
	}
}
