#ifndef WPCOMMENTLIST_H
#define WPCOMMENTLIST_H

#include <QWidget>
#include <QList>
#include "MDragWidget.h"

class WPCommentListItem;
class WPDataComment;
class WPNetworkEngine;
class QVBoxLayout;
class WPScrollArea;
class PopupMenu;

class WPCommentList : public MDragWidget
{
	Q_OBJECT

public:
	WPCommentList(QWidget* aParent, const QList<WPDataComment>& aData,
			WPNetworkEngine& aNetworkEngine, WPScrollArea& aScrollArea,
			bool aShowFullComment = false);
	~WPCommentList();

public:
	bool MDragWidgetIsDragging() const;
	QSize VisibleSize() const;
	void setFocus();

signals:
	void ReplyToComment(const WPDataComment& aComment);
	
public slots:
	void GetGravatarCompleted(const QString& aEmail, const QString& aFilename);
	void ShowContextMenu(WPCommentListItem* aItem);
	
private slots:
    void MenuHiding();
	void Approve();
	void Unapprove();
	void View();
	void Reply();
	void Delete();
	void Spam();

private:
	void AddComment(const WPDataComment& aComment, bool aShowFullComment);
	
private: // Data
	const QList<WPDataComment>& iData;
	WPNetworkEngine& iNetworkEngine;
	WPScrollArea& iScrollArea;

	QList<WPCommentListItem*> iItems;
	WPCommentListItem* iCurrentItem;
	QVBoxLayout* iLayout;
	
	QAction* iActionApprove;
	QAction* iActionUnapprove;
	QAction* iActionView;
	QAction* iActionHide;
	QAction* iActionReply;
	QAction* iActionDelete;
	QAction* iActionSpam;
	
	PopupMenu* iPopupMenu;
};

#endif // WPCOMMENTLISTITEM_H
