#ifndef WPSCROLLAREA_H
#define WPSCROLLAREA_H

#include <QVBoxLayout>
#include <QScrollArea>
#include <QTime>

class QTimer;

class WPScrollArea : public QScrollArea
{
    Q_OBJECT

public:
	WPScrollArea(QWidget* parent = 0);
    ~WPScrollArea();
    
signals:
	void DragStarted();
    
public slots:
	void Deaccelerate();
	
public:
	bool IsDragging() const;
	void addWidget(QWidget* aWidget);
	void removeWidget(QWidget* aWidget);
	void setWidget(QWidget* aWidget);
	bool focusNextPrevChild(bool next);
	bool eventFilter(QObject *, QEvent *);
	void InitLayout();
	void AddStretch();
	void ScrollToTop();
	QLayoutItem* itemAt(int aIndex);
	QSize GetVisibleContentSize() const;
	QPoint TopRightInGlobal() const;
	void DisableScrollAtWidget(QWidget* aWidget);
    void RemoveSelfScrollingWidget(QWidget* aWidget);

protected:
	void mousePressEvent(QMouseEvent* aEvent);
	void mouseMoveEvent(QMouseEvent* aEvent);
	void mouseReleaseEvent(QMouseEvent* aEvent);

private:
	void HandleScrollEvent(QKeyEvent* aEvent);

private:
	int iPreviousY;
	bool iDragging;
	
	QWidget *iParent;
	QTime iDecelerationStarted;
	QTime iTimeStamp;
	int iSpeed;
	QTimer* iTimer;
	
	QFrame* iContent;
	QVBoxLayout* iLayout;

	QList<QWidget*> iScrollDisableWidgets;
};

#endif // WPSCROLLAREA_H
