
#include <QFrame>

#include "BlogView.h"
#include "writepageview.h"
#include "WPNetworkEngine.h"

WritePageView::WritePageView(BlogView *parent) : QFrame(parent), iParent(parent)
{
#ifdef Q_WS_MAEMO_5
	setWindowFlags(Qt::Window);
	setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
	ui.setupUi(this);
	ui.scrollArea->setWidget(ui.layoutWidget);
	ui.layoutWidget->setContentsMargins(5, 5, 5, 5);
	ui.titleEditor->SetExplanation(tr("Page title"));
	ui.textBrowser->setExplanationText(tr("Tap to begin writing"));
	
#ifdef Q_OS_SYMBIAN
	// Prepare softkeys
	QAction* leftSoftKey = new QAction(tr("Save"), this);
	connect(leftSoftKey, SIGNAL(triggered()), this, SLOT(SavePage()));
	leftSoftKey->setSoftKeyRole(QAction::PositiveSoftKey);
	addAction(leftSoftKey);
	QAction* rightSoftKey = new QAction(tr("Cancel"), this);
	connect(rightSoftKey, SIGNAL(triggered()), iParent, SLOT(ShowTabbedView()));
	rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
	addAction(rightSoftKey);
#else
	QPushButton * saveButton = new QPushButton(tr("Save"));
	saveButton->setObjectName("Maemo5NavigationButton");
	connect(saveButton, SIGNAL(clicked()), this, SLOT(SavePage()));
	QHBoxLayout * buttonLayout = new QHBoxLayout;
	buttonLayout->addWidget(saveButton);
#ifndef Q_WS_MAEMO_5
	QPushButton * cancelButton = new QPushButton(tr("Cancel"));
	cancelButton->setObjectName(QString::fromUtf8("Maemo5NavigationButton"));
	connect(cancelButton, SIGNAL(clicked()), iParent, SLOT(ShowTabbedView()));
	buttonLayout->addWidget(cancelButton);
#endif
	ui.verticalLayout->addLayout(buttonLayout);
#endif

	connect(ui.addPhotosButton, SIGNAL(clicked(bool)), this, SLOT(AddPhotosClicked(bool)));
	
	ui.scrollArea->DisableScrollAtWidget(ui.statusEditor->view());
}

WritePageView::~WritePageView()
{
}

void WritePageView::NewPage(WPDataBlog& aBlog)
{
	iBlog = &aBlog;
	iPageId = "";
	iAddedImages.clear();
	ui.titleEditor->setText("");
        ui.textBrowser->reset();
	ui.textBrowser->setText("");
	ui.dateTimeEditor->setDateTime(QDateTime::currentDateTime().toLocalTime());
	ui.addPhotosButton->setText(tr("Add photos") + "...");
	// Statuses
	ui.statusEditor->clear();
	QStringList keyList, valueList;
	QMap<QString, QString>::const_iterator i;
	for (i = iBlog->iPageStatuses.constBegin(); i != iBlog->iPageStatuses.constEnd(); i++)
	{
		keyList.append(i.key());
		valueList.append(i.value());
	}
	ui.statusEditor->addItems(valueList);
	ui.statusEditor->addItem("Local draft");
	ui.titleEditor->setFocus();
}

void WritePageView::EditPage(WPDataBlog& aBlog, const QString& aPageId)
{
	iBlog = &aBlog;
	iPageId = aPageId;
	iAddedImages.clear();
	WPDataPage page;
	// TODO it would be better to handle page selection in blog view
	const int pageCount = iBlog->iPages.count();
	for (int i = 0; i < pageCount; ++i)
	{
		if (iBlog->iPages.at(i).iPageId.compare(aPageId) == 0)
		{
			page = iBlog->iPages.at(i);
			break;
		}
	}
	
	const int localDraftCount = iBlog->iLocalDraftPages.count();
	for (int i = 0; i < localDraftCount; ++i)
	{
		if (iBlog->iLocalDraftPages.at(i).iPageId.compare(aPageId) == 0)
		{
			page = iBlog->iLocalDraftPages.at(i);
			break;
		}
	}

	ui.titleEditor->setText(page.iTitle);
        ui.textBrowser->reset();
	ui.textBrowser->setText(page.iDescription);
	ui.dateTimeEditor->setDateTime(QDateTime::fromString(page.iDateCreated, "yyyyMMddTHH:mm:ss"));
	ui.addPhotosButton->setText(tr("Add photos") + "...");
	
	// Statuses
	ui.statusEditor->clear();
	QStringList keyList, valueList;
	QMap<QString, QString>::const_iterator i;
	for (i = iBlog->iPageStatuses.constBegin(); i != iBlog->iPageStatuses.constEnd(); i++)
	{
		keyList.append(i.key());
		valueList.append(i.value());
	}
	ui.statusEditor->addItems(valueList);
	if (WPDataPage::IsLocalDraftId(iPageId))
		{
		ui.statusEditor->addItem("Local draft");
		ui.statusEditor->setCurrentIndex(ui.statusEditor->count() - 1);
		}
	// Using regular for loop as we need to get the index
	for (int i = 0; i < keyList.count(); ++i)
	{
		if (page.iPageStatus.compare(keyList.at(i)) == 0)
			ui.statusEditor->setCurrentIndex(i);
	}
	ui.titleEditor->setFocus();
}

void WritePageView::SavePage()
{
	const WPDataPage* page = NULL;
	
	if (ui.statusEditor->currentText() == "Local draft")
	{
		if (iPageId.isEmpty())
		{
			WPDataPage page;
			QString id = "local_";
			id += QString::number(qrand());
			page.iPageId = id;
			page.iTitle = ui.titleEditor->text();
			page.iDescription = ui.textBrowser->text();
			page.iDateCreated = WPNetworkEngine::ISO8601Time(ui.dateTimeEditor->dateTime());
			iBlog->iLocalDraftPages.append(page);
		}
		else
		{
			WPDataPage* page = iParent->GetNetworkEngine()->GetLocalDraftPage(iPageId);
			if (page)
			{
				page->iTitle = ui.titleEditor->text();
				page->iDescription = ui.textBrowser->text();
				page->iDateCreated = WPNetworkEngine::ISO8601Time(ui.dateTimeEditor->dateTime());
			}
		}
		emit UpdatePageData(*iBlog);
		iParent->ShowTabbedView();
#ifdef Q_WS_MAEMO_5
		close();
#endif
		return;
	}
	
	const int pageCount = iBlog->iPages.size();
	for (int i = 0; i < pageCount; i++)
	{
		if(iBlog->iPages.at(i).iPageId.compare(iPageId) == 0)
		{
			page = &iBlog->iPages.at(i);
			break;
		}
	}
	
	// Check if content has changed
	if (!page || ui.textBrowser->text() != page->iDescription
			|| ui.titleEditor->text() != page->iTitle
			|| ui.statusEditor->currentText() != iBlog->iPageStatuses.value(page->iPageStatus)
			|| iAddedImages.size() > 0)
	{
		QMap<QString, QString> content;
		content.insert("title", ui.titleEditor->text());
		content.insert("description", ui.textBrowser->text());
		content.insert("page_status", iBlog->iPageStatuses.key(ui.statusEditor->currentText()));
		content.insert("dateCreated", WPNetworkEngine::ISO8601Time(ui.dateTimeEditor->dateTime().toUTC()));
		
		// Argument "publish" seems to dictate whether to publish or edit a page in WP API.
		// There's also "methodName" to achieve that -- maybe this is redundant?
		if (iPageId.isEmpty() || WPDataPage::IsLocalDraftId(iPageId))
		{
			emit NewPage(content, "1", iAddedImages, iPageId);
		}
		else
		{
			emit EditPage(iPageId, content, "0", iAddedImages);
		}
		
		iParent->ShowTabbedView();
#ifdef Q_WS_MAEMO_5
		close();
#endif
	}
}

void WritePageView::AddPhotosClicked(bool /*aDummy*/)
{
	iParent->ShowImageList(iAddedImages, false);
}

void WritePageView::ImagesAdded(const QStringList& aFilenames)
{
	iAddedImages = aFilenames;
	
	QString labelText;
	if (iAddedImages.size() == 0)
		labelText = tr("Add photos") + "...";
	else if (iAddedImages.size() == 1) 
		labelText = tr("Adding") + " 1 " + tr("new photo"); 
	else
		labelText = tr("Adding") + " " + QString::number(iAddedImages.size()) + " " + tr("new photos");
	
	ui.addPhotosButton->setText(labelText);
}

void WritePageView::setFocus()
{
	ui.scrollArea->ScrollToTop();
	ui.scrollArea->setFocus();
}
